<?php

if (!defined('ABSPATH')) { exit; }

if (!class_exists('DHLPWC_Model_Service_Track_Trace')) :

class DHLPWC_Model_Service_Track_Trace extends DHLPWC_Model_Core_Singleton_Abstract
{

    const QUERY_LANDCODE = 'lang';

    protected $url = 'https://my.dhlparcel.nl/home/tracktrace/';
    protected $alternate_urls = [
        'BE' => 'https://my.dhlparcel.be/home/tracktrace/'
    ];

    public function get_url($tracking_code = null, $locale = null, $country_code = null)
    {
        $query_args = array();
        $appendix = '';
        if ($tracking_code !== null) {
            $appendix = urlencode($tracking_code);
        }

        if ($locale !== null) {
            $query_args[self::QUERY_LANDCODE] = 'en-NL';
            if (substr(urlencode($locale), 0, 2) === 'nl') {
                $query_args[self::QUERY_LANDCODE] = 'nl-NL';
            }
        }

        $tracking_url = $this->url;
        if (array_key_exists(strtoupper($country_code), $this->alternate_urls)) {
            $tracking_url = $this->alternate_urls[strtoupper($country_code)];
        }

        return add_query_arg($query_args, $tracking_url . $appendix);
    }

    public function get_track_trace_from_order($order_id)
    {
        $service = DHLPWC_Model_Service_Order_Meta::instance();
        $labels = $service->get_labels($order_id);

        if (!$labels || !is_array($labels)) {
            return array();
        }

        $tracker_codes = array();
        foreach($labels as $label) {
            if (array_key_exists('tracker_code', $label) && empty($label['is_return'])) {
                $tracker_codes[] = $label['tracker_code'];
            }
        }

        return $tracker_codes;
    }

}

endif;
